package MN::CGI; # class to manage MyNews computer graphic interface

use strict;
use CGI;
use HTTP::Lite;

use Exporter;
@MN::CGI::ISA = qw(Exporter);
@MN::CGI::EXPORT = qw(__ _c_);

my $current;
my $header = 0;
my $menu = 0;
my $user;
my $i18n = {};
my @navitems = ();
my $console = "";
my $currentlang;

sub getCurrent # get current cgi
{
    return $current;
}
sub getUser # get current user
{
    return $user;
}

sub new # constructor
{
    my $class_name = shift;

    my $cgi;
    if ($ENV{DEBUG})
    {
	$cgi = new CGI($ENV{QUERY_STRING});
    }
    else
    {
	$cgi = new CGI();
    }

    my $self = $cgi->Vars();
    
    bless $self, $class_name;

    $current = $self;

    $console = "";  # reset CGI console

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $lang = "en";
    if ($ENV{REMOTE_USER} && ($user = MN::User->getObject($ENV{REMOTE_USER})))
    {
	if ($user->{lang})
	{
	    $lang = $user->{lang};
	}
	$user->{admin} = MN::UserGroup->getObject($user->{login}, "admin");
    }
    elsif ($mne->{conf}->{publang}) # forced lang
    {
	$lang = $mne->{conf}->{publang};
    }
    else
    {
	$lang = $self->prefLang(); # lang from browser
    }
    $self->setLang($lang);

    if (!$ENV{REMOTE_USER}) # public area
    {
	if ($mne->{conf}->{puburl})
	{
	    $mne->{conf}->{privurl} = $mne->{conf}->{weburl};
	    $mne->{conf}->{weburl} = $mne->{conf}->{puburl};
	}
	if ($mne->{conf}->{pubdir})
	{
	    $mne->{conf}->{privdir} = $mne->{conf}->{webdir};
	    $mne->{conf}->{webdir} = $mne->{conf}->{pubdir};
	}
    }

    return $self; # new CGI object
}

sub setLang # force CGI to reload a language
{
    my $self = shift;
    my $lang = shift; # language to reload

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir} && open(MSG, "$mne->{conf}->{webdir}/lang/$lang/messages"))
    {
	my $line;
	my ($key, $val);
	while ($line = <MSG>)
	{
	    chomp($line);
	    ($key, $val) = split(/\t+/, $line);
	    if ($val)
	    {
		$i18n->{$key} = $val;
	    }
	}
	close(MSG);
    }
    $currentlang = $lang;
}

sub prefLang # get preferred language from browser
{
    my @alangs = split(/,/, $ENV{HTTP_ACCEPT_LANGUAGE});
    my $alang;
    foreach $alang (@alangs)
    {
	$alang =~ s/;.*//;
	if ($alang =~ /^fr/i)
	{
	    return "fr";
	}
	elsif ($alang =~ /^(en|us)/i)
	{
	    return "en";
	}
    }
    return "en";
}

sub __ # i18n function
{
    my $msg = shift;
    
    return $i18n->{$msg} || "*** $msg ***";
}

sub getLangName # return langage name(s)
{
    my $self = shift;
    my $lang = shift; # optional lang code;

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $file;
    my $name = "";

    if ($lang)
    {
	if ($mne->{conf}->{webdir} && open(NAME, "$mne->{conf}->{webdir}/lang/$lang/name"))
	{
	    $name = <NAME>;
	    chomp($name);
	    close(NAME);
	}
	return $name;
    }
    else
    {
	my $langs = {};
	if ($mne->{conf}->{webdir})
	{
	    foreach $file (`ls $mne->{conf}->{webdir}/lang/*/name`)
	    {
		chomp($file);
		$name = "";
		if (open(NAME, $file))
		{
		    $name = <NAME>;
		    chomp($name);
		    close(NAME);
		}
		if ($file =~ /(\w+)\/name$/)
		{
		    $langs->{$1} = $name;
		}
	    }
	}
	return $langs; # all language names in a hash table
    }
 
}

sub langSelectorCode # return HTML code for the lang selector
{
    my $self = shift;
    my $selected = shift; # optional default selected language
    my $submit = shift; # 1 to submit on value change
    my $css = shift; # optional css class prefix
    my $name = shift || "lang"; # optional selector name ("lang" by default)

    my $langs = $self->getLangName();

    my $key;
    my $code = '<select class="'.$css.'formselect" name="'.$name.'"';
    if ($submit)
    {
	$code .= ' onchange="form.submit();"';
    }
    $code .= '>'."\n";
    foreach $key (sort {$langs->{$a} cmp $langs->{$b}} (keys(%{$langs})))
    {
	$code .= '<option value="'.$key.'"';
	if ($key eq $selected)
	{
	    $code .= " selected";
	}
	$code .= ">$langs->{$key}\n";
    }
    $code .= "</select>\n";

    return $code;
}

sub headerCode # return HTML code for the header
{
    my $self = shift;
    my $title = "";
    my $str;
    my $url;
    my $mne = MN::Engine->getCurrent(); # MyNews Engine

    @navitems = ({"name" => "MyNews", "url" => $mne->{conf}->{weburl}});

    while ($str = shift)
    {
	$url = shift;
	$title .= " &gt; $str";
	push @navitems, {"name" => $str, "url" => "$mne->{conf}->{weburl}$url"};
    }

    my $code = "Content-type: text/html\n\n";
    $code .= "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
    $code .= "<html>\n";
    $code .= "<head>\n";
    $code .= "<title>MyNews$title</title>\n";
    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/default.css\" type=\"text/css\" media=\"screen\">\n";
    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/print.css\" type=\"text/css\" media=\"print\">\n";
    if ($user)
    {
	if ($user->{testtheme} && -e "$mne->{conf}->{webdir}/css/themes/$user->{login}.css")
	{
	    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/themes/$user->{login}.css\" type=\"text/css\" media=\"screen\">\n";
	}
	elsif ($user->{theme} && -e "$mne->{conf}->{webdir}/css/themes/$user->{theme}/theme.css")
	{
	    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/themes/$user->{theme}/theme.css\" type=\"text/css\" media=\"screen\">\n";
	}
	if ($user->{theme} && -e "$mne->{conf}->{webdir}/css/themes/$user->{theme}/print.css")
	{
	    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/themes/$user->{theme}/print.css\" type=\"text/css\" media=\"print\">\n";
	}
    }
    else
    {
	if ($mne->{conf}->{pubtheme} && -e "$mne->{conf}->{webdir}/css/themes/$mne->{conf}->{pubtheme}/theme.css")
	{
	    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/themes/$mne->{conf}->{pubtheme}/theme.css\" type=\"text/css\" media=\"screen\">\n";
	}
	if ($mne->{conf}->{pubtheme} && -e "$mne->{conf}->{webdir}/css/themes/$mne->{conf}->{pubtheme}/print.css")
	{
	    $code .= "<link rel=\"StyleSheet\" href=\"$mne->{conf}->{weburl}/css/themes/$mne->{conf}->{pubtheme}/print.css\" type=\"text/css\" media=\"print\">\n";
	}
    }
    $code .= "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n";
    $code .= "</head>\n";
    $code .= '<body class="body"';
    if ($self->{onload})
    {
	$code .= " onload='$self->{onload}'";
    }
    $code .= ">\n";

    if ($mne->{conf}->{headerurl})
    {
	$code .= $self->insertCode($mne->{conf}->{headerurl});
    }

    $header = 1;

    return $code;
}

sub menuCode # return HTML code for the menu
{
    my $self = shift;
    
    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my %selected = ();
    
    if ($ENV{SCRIPT_NAME} && $ENV{SCRIPT_NAME} =~ /admin\/[^\/]+\.cgi*/)
    {
	$selected{"admin"} = "selected";
    }

    if ($ENV{SCRIPT_NAME} && $ENV{SCRIPT_NAME} =~ /([^\/]+)\.cgi$/)
    {
	$selected{$1} = "selected";
    }

    my $code = "";
    $code .= '<table class="main" border=0 cellspacing=0 cellpadding=0>'."\n";
    my $hidden = "";
    if ($user)
    {
	$hidden = $user->isHidden("Navigation bar");
    }
    else
    {
	$hidden = $mne->isHidden("Navigation bar");
    }
    $code  .= "<tr class=\"navbar$hidden\">\n";
    $code  .= "<td class=\"navbar$hidden\" colspan=2>\n";
    $code  .= "<table class=\"navbartb\" border=0 cellspacing=0 cellpadding=0 width=\"100%\">\n";
    $code  .= "<tr class=\"navbartr\">\n";
    $code  .= "<td class=\"navbarleft\">\n";
    $code  .= "</td>\n";
    $code  .= "<td class=\"navbartd\" width=\"100%\">\n";
    my $navitem;
    my $li = "";
    my $i = 1;
    my $nbi = scalar(@navitems);
    foreach $navitem (@navitems)
    {
	if ($i >= $nbi) { $li = "2" };
	$code .= "<div class=\"navitem$li\"><a class=\"navitem$li\" href=\"$navitem->{url}\">$navitem->{name}</a></div>\n";
	$i++;
    }
    if ($user)
    {
	$code .= '<div class="navitem3">'."\n";
	if ($mne->{conf}->{puburl})
	{
	    $code .= '<a class="navitem3" href="'.$mne->{conf}->{puburl}.'" title="'.__("Public area").'">';
	}
	$code .= $user->getLongName();
	if ($mne->{conf}->{puburl})
	{
	    $code .= '</a>';
	}
	$code .= "</div>\n";
    }
    else # public area
    {
	if ($mne->{conf}->{privurl})
	{
	    $code .= '<div class="navitem3"><a class="navitem3" href="'.$mne->{conf}->{privurl}.'" title="'.__("Private area").'">'.__("Login").'</a></div>'."\n";

	}
    }
    $code .= "</td>\n";
    $code  .= "<td class=\"navbarright\">\n";
    $code  .= "</td>\n";
    $code .= "</tr>\n";
    $code .= "</table>\n";
    $code .= "</td>\n";
    $code .= "</tr>\n";
    $code .= '<tr class="maintr">'."\n";
    $code .= '<td class="menutd">'."\n";

    # logo
    my $msg;
    if ($user)
    {
	$msg = __("News from my favourite websites");
    }
    else
    {
	$msg = __("Selected news from the web");
    }
    $code .= '<a href="'.$mne->{conf}->{weburl}.'" class="logo"><div class="logo" title="'.$msg.'"></div></a>'."\n";

    # search
    my $sparam = "";
    if ($selected{'search'})
    {
	if ($self->{topic})
	{
	    $selected{'topic'} = "selected";
	    $self->{id} = $self->{topic};
	}
	elsif ($self->{template})
	{
	    $selected{'template'} = "selected";
	    $self->{id} = $self->{template};
	}
	elsif ($self->{other})
	{
	    $selected{'other'} = "selected";
	}
    }
    if ($selected{'topic'})
    {
	$sparam = '?topic='.$self->{id};
    }
    elsif ($selected{'template'})
    {
	$sparam = '?template='.$self->{id};
    }
    elsif ($selected{'other'})
    {
	$sparam = '?other=1';
    }
    elsif ($selected{'news'} && $user && $self->{id} =~ /^([^\_]+)/)
    {
	my $sb = MN::Subscr->getObject($1, $user->{login});
	if (!$sb)
	{
	    $sparam = '?other=1';
	    $selected{'other'} = "selected";
	}
    }
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/search.cgi'.$sparam.'" class="'.$selected{'search'}.'menuitem"><div class="'.$selected{'search'}.'menuitem" title="'.__("News search by keywords").'">'.__("Search").'</div></a>'."\n";

    if ($user)
    {
	if ($selected{'template'})
	{
	    my $sb = MN::Subscr->getObject($self->{id}, $user->{login});
	    if (!$sb)
	    {
		$selected{'other'} = "selected";
	    }
	}
	elsif ($selected{'topic'} && $self->{id} =~ /^([^\_]+)/)
	{
	    my $sb = MN::Subscr->getObject($1, $user->{login});
	    if (!$sb)
	    {
		$selected{'other'} = "selected";
	    }
	}

	# other sites
	$code .= '<a href="'.$mne->{conf}->{weburl}.'/other.cgi" class="'.$selected{'other'}.'menuitem"><div class="'.$selected{'other'}.'menuitem" title="'.__("News from other websites").'">'.__("Other sites").'</div></a>'."\n";
	
	# prefs
	$code .= '<a href="'.$mne->{conf}->{weburl}.'/prefs.cgi" class="'.$selected{'prefs'}.'menuitem"><div class="'.$selected{'prefs'}.'menuitem" title="'.__("My preferences (name, email, password, language, theme...)").'">'.__("Preferences").'</div></a>'."\n";
	
	if ($user->{admin})
	{
	    # admin
	    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/" class="'.$selected{'admin'}.'menuitem"><div class="'.$selected{'admin'}.'menuitem" title="'.__("MyNews website administration").'">'.__("Admin").'</div></a>'."\n";
	}
    }
    $code .= '<hr class="separator">'."\n";

    if ($selected{'template'}) # template
    {
	my $tmpl = MN::Template->getTemplate($self->{id});
	$code .= '<table border=0 cellpadding=0 cellspacing=0 width="100%" class="netemetb">'."\n";

	$code.= "<caption class=\"netemetitle\">\n";
	if ($user && !$selected{'other'})
	{
	    $code .= MN::__("Subscription");
	}
	else
	{
	    $code .= MN::__("Website");
	}
	$code .=  "</caption>\n";

	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemetopleft"></td>'."\n";
	$code .= '<td class="netemetop"></td>'."\n";
	$code .= '<td class="netemetopright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemeleft"></td>'."\n";
	$code .= '<td class="netemetd">'."\n";
	$code .= "<div class=\"newstemplatesmenu\">\n";
	$code .= $tmpl->getCode("menu");
	$code .= "</div>\n";
	$code .= "</td>\n";
	$code .= '<td class="netemeright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemebottomleft"></td>'."\n";
	$code .= '<td class="netemebottom"></td>'."\n";
	$code .= '<td class="netemebottomright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= "</table>\n";
	if ($user)
	{
	    $code .= '<a href="template.cgi?id='.$self->{id}.'&swsub=1" class="menuitem">'."\n";
	    $code .= '<div class="menuitem"';
	    if ($selected{'other'})
	    {
		$code .= ' title="'.sprintf(__("Subscribe to %s"), $tmpl->{name}).'">'.__("Subscribe")."\n";
	    }
	    else
	    {
		$code .= ' title="'.sprintf(__("Unsubscribe from %s"), $tmpl->{name}).'">'.__("Unsubscribe")."\n";
	    }
	    $code .= "</div>\n";
	    $code .= "</a>\n";
	}

	$code .= '<hr class="separator">'."\n";

	my @topics = $tmpl->getTopics();

	if (scalar(@topics))
	{
	    $code .= '<table border=0 cellpadding=0 cellspacing=0 width="100%" class="netometb">'."\n";

	    $code.= "<caption class=\"netometitle\">\n";
	    $code .= MN::__("Topics");
	    $code .=  "</caption>\n";

	    $code .= '<tr class="netometr">'."\n";
	    $code .= '<td class="netometopleft"></td>'."\n";
	    $code .= '<td class="netometop"></td>'."\n";
	    $code .= '<td class="netometopright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= '<tr class="netometr">'."\n";
	    $code .= '<td class="netomeleft"></td>'."\n";
	    $code .= '<td class="netometd">'."\n";
	    $code .= "<div class=\"newstopicsmenu\">\n";
	    my $topic;
	    foreach $topic (@topics)
	    {
		$code .= $topic->getCode("menu");
	    }
	    $code .= "</div>\n";
	    $code .= "</td>\n";
	    $code .= '<td class="netomeright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= '<tr class="netometr">'."\n";
	    $code .= '<td class="netomebottomleft"></td>'."\n";
	    $code .= '<td class="netomebottom"></td>'."\n";
	    $code .= '<td class="netomebottomright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= "</table>\n";
	}
    }
    elsif ($selected{'topic'}) # topic
    {
	my $topic = MN::Topic->getTopic($self->{id});
	my $tmpl = $topic->getTemplate();

	$code .= '<table border=0 cellpadding=0 cellspacing=0 width="100%" class="netemetb">'."\n";

	$code.= "<caption class=\"netemetitle\">\n";
	if ($user && !$selected{'other'})
	{
	    $code .= MN::__("Subscription");
	}
	else
	{
	    $code .= MN::__("Website");
	}
	$code .=  "</caption>\n";

	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemetopleft"></td>'."\n";
	$code .= '<td class="netemetop"></td>'."\n";
	$code .= '<td class="netemetopright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemeleft"></td>'."\n";
	$code .= '<td class="netemetd">'."\n";
	$code .= "<div class=\"newstemplatesmenu\">\n";
	$code .= $tmpl->getCode("menu");
	$code .= "</div>\n";
	$code .= "</td>\n";
	$code .= '<td class="netemeright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netemetr">'."\n";
	$code .= '<td class="netemebottomleft"></td>'."\n";
	$code .= '<td class="netemebottom"></td>'."\n";
	$code .= '<td class="netemebottomright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= "</table>\n";

	$code .= '<hr class="separator">'."\n";

	$code .= '<table border=0 cellpadding=0 cellspacing=0 width="100%" class="netometb">'."\n";

	$code.= "<caption class=\"netometitle\">\n";
	$code .= MN::__("Topic");
	$code .=  "</caption>\n";

	$code .= '<tr class="netometr">'."\n";
	$code .= '<td class="netometopleft"></td>'."\n";
	$code .= '<td class="netometop"></td>'."\n";
	$code .= '<td class="netometopright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netometr">'."\n";
	$code .= '<td class="netomeleft"></td>'."\n";
	$code .= '<td class="netometd">'."\n";
	$code .= "<div class=\"newstopicsmenu\">\n";
	$code .= $topic->getCode("menu");
	$code .= "</div>\n";
	$code .= "</td>\n";
	$code .= '<td class="netomeright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= '<tr class="netometr">'."\n";
	$code .= '<td class="netomebottomleft"></td>'."\n";
	$code .= '<td class="netomebottom"></td>'."\n";
	$code .= '<td class="netomebottomright"></td>'."\n";
	$code .= "</tr>\n";
	$code .= "</table>\n";
    }
    elsif ($selected{'admin'}) # admin menu
    {
	$code .= $self->adminMenuCode();
    }
    else
    {
	my $tmpls;
	my $tmpl;
	my $subs = 0;
	if ($user)
	{
	    if ($selected{'other'})
	    {
		$tmpls = $user->getUnsubTemplates();
	    }
	    else
	    {
		$tmpls = $user->getSubTemplates();
		$subs = 1;
	    }
	}
	else
	{
	    my @tmpls = MN::Template->getObjects("active = 1", "id");
	    foreach $tmpl (@tmpls)
	    {
		MN::Template::getCurrent()->{$tmpl->{id}} = $tmpl;
	    }
	    $tmpls = \@tmpls;
	}
	if (scalar(@{$tmpls}))
	{
	    $code .= '<table border=0 cellpadding=0 cellspacing=0 width="100%" class="netemetb">'."\n";

	    $code.= "<caption class=\"netemetitle\">\n";
	    if ($subs)
	    {
		$code .= MN::__("Subscriptions");
	    }
	    else
	    {
		$code .= MN::__("Websites");
	    }
	    $code .=  "</caption>\n";
	    
	    $code .= '<tr class="netemetr">'."\n";
	    $code .= '<td class="netemetopleft"></td>'."\n";
	    $code .= '<td class="netemetop"></td>'."\n";
	    $code .= '<td class="netemetopright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= '<tr class="netemetr">'."\n";
	    $code .= '<td class="netemeleft"></td>'."\n";
	    $code .= '<td class="netemetd">'."\n";
	    $code .= "<div class=\"newstemplatesmenu\">\n";
	    foreach $tmpl (@{$tmpls})
	    {
		$code .= $tmpl->getCode("menu");
	    }
	    $code .= "</div>\n";
	    $code .= "</td>\n";
	    $code .= '<td class="netemeright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= '<tr class="netemetr">'."\n";
	    $code .= '<td class="netemebottomleft"></td>'."\n";
	    $code .= '<td class="netemebottom"></td>'."\n";
	    $code .= '<td class="netemebottomright"></td>'."\n";
	    $code .= "</tr>\n";
	    $code .= "</table>\n";
	}
    }

    $code .= '<a href="'.$mne->{conf}->{weburl}.'/lang/'.$currentlang.'/index.cgi" class="doc"><div class="doc" title="'.MN::__("Documentation").'">MyNews '.$MN::VERSION.'</div></a>'."\n";

    $code .= "</td>\n";
    $code .= '<td class="maintd">'."\n";

    $menu = 1;

    return $code;
}

sub adminMenuCode # return HTML code for the admin menu
{
    my $self = shift;
    
    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my %selected = ();
    
    if ($ENV{SCRIPT_NAME} && $ENV{SCRIPT_NAME} =~ /([^\/]+)\.cgi*/)
    {
	$selected{$1} = "selected";
    }

    my $code = "";

    # templates
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/templates.cgi" class="'.$selected{'templates'}.'adminmenuitem"><div class="'.$selected{'templates'}.'adminmenuitem" title="'.__("Grabbed websites description").'">'.__("Templates").'</div></a>'."\n";
    # topics
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/topics.cgi" class="'.$selected{'topics'}.'adminmenuitem"><div class="'.$selected{'topics'}.'adminmenuitem" title="'.__("Website topics with or without icon").'">'.__("Topics").'</div></a>'."\n";

    # news
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/news.cgi" class="'.$selected{'news'}.'adminmenuitem"><div class="'.$selected{'news'}.'adminmenuitem" title="'.__("Grabbed news list").'">'.__("News").'</div></a>'."\n";

    # themes
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/themes.cgi" class="'.$selected{'themes'}.'adminmenuitem"><div class="'.$selected{'themes'}.'adminmenuitem" title="'.__("Graphical themes (colors, images, CSS...)").'">'.__("Themes").'</div></a>'."\n";

    # users
    $code .= '<a href="'.$mne->{conf}->{weburl}.'/admin/users.cgi" class="'.$selected{'users'}.'adminmenuitem"><div class="'.$selected{'users'}.'adminmenuitem" title="'.__("Users authorized to access MyNews").'">'.__("Users").'</div></a>'."\n";

    return $code;
}

sub footerCode # return HTML code for the footer
{
    my $self = shift;
    my $code = "";
    my $mne = MN::Engine->getCurrent(); # MyNews Engine

    if ($menu)
    {
	if ($mne->{conf}->{innerurl})
	{
	    $code .= $self->insertCode($mne->{conf}->{innerurl})
	}
	$code .= "</td>\n";
	$code .= "</tr>\n";
	$code .= "</table>\n";
    }
    if ($console && $user && !$user->isHidden("Console"))
    {
	$code .= "<table border=0 cellspacing=0 cellpadding=0 class=\"consoletb\" width=\"100%\">\n";
	$code.= "<caption class=\"consoletitle\">\n";
	$code .= MN::__("Console");
	$code .=  "</caption>\n";
	$code .= "<tr class=\"consoletr\">\n";
	$code .= "<td class=\"consoletopleft\"></td>\n";
	$code .= "<td class=\"consoletop\"></td>\n";
	$code .= "<td class=\"consoletopright\"></td>\n";
	$code .= "</tr>\n";
	$code .= "<tr class=\"consoletr\">\n";
	$code .= "<td class=\"consoleleft\"></td>\n";
	$code .= "<td class=\"consoletd\">\n";
	$code .= "<form>\n";
	$code .= "<textarea class=\"console\" rows=10 cols=80>".$console."</textarea>\n";
	$code .= "</form>\n";
	$code .= "</td>\n";
	$code .= "<td class=\"consoleright\"></td>\n";
	$code .= "</tr>\n";
	$code .= "<tr class=\"consoletr\">\n";
	$code .= "<td class=\"consolebottomleft\"></td>\n";
	$code .= "<td class=\"consolebottom\"></td>\n";
	$code .= "<td class=\"consolebottomright\"></td>\n";
	$code .= "</tr>\n";
	$code .= "</table>\n";

    }
    if ($mne->{conf}->{footerurl})
    {
	$code .= $self->insertCode($mne->{conf}->{footerurl});
    }
    $code .= "</body>\n";
    $code .= "</html>\n";

    return $code;
}

sub unquote # unquote string to be used in HTML code
{
    my $self = shift;
    my $msg = shift; 

    $msg =~ s/&/&amp;/g;
    $msg =~ s/\"/&quot;/g;
    $msg =~ s/\'/&\#39;/g;
    $msg =~ s/</&lt;/g;
    $msg =~ s/>/&gt;/g;
    return $msg; 
}

sub code64
{
    my $c = shift;

    my $v = unpack("C", $c);
    $v += 64;
    return sprintf("%c", $v);
}

sub unutf # replace UTF chars by ISO-8859-1 chars 
{
    my $self = shift;
    my $msg = shift; 

    $msg =~ s//&aacute;/g;
    $msg =~ s//&agrave;/g;
    $msg =~ s//&acirc;/g;
    $msg =~ s//&auml;/g;
    $msg =~ s//&eacute;/g;
    $msg =~ s//&egrave;/g;
    $msg =~ s//&ecirc;/g;
    $msg =~ s//&euml;/g;
    $msg =~ s//&iacute;/g;
    $msg =~ s//&igrave;/g;
    $msg =~ s//&icirc;/g;
    $msg =~ s//&iuml;/g;
    $msg =~ s//&oacute;/g;
    $msg =~ s//&ograve;/g;
    $msg =~ s//&ocirc;/g;
    $msg =~ s//&ouml;/g;
    $msg =~ s//&uacute;/g;
    $msg =~ s//&ugrave;/g;
    $msg =~ s//&ucirc;/g;
    $msg =~ s//&uuml;/g;
    $msg =~ s//&ccedil;/g;
    $msg =~ s//&Aacute;/g;
    $msg =~ s//&Agrave;/g;
    $msg =~ s//&Acirc;/g;
    $msg =~ s//&Auml;/g;
    $msg =~ s//&Eacute;/g;
    $msg =~ s//&Egrave;/g;
    $msg =~ s//&Ecirc;/g;
    $msg =~ s//&Euml;/g;
    $msg =~ s//&Iacute;/g;
    $msg =~ s//&Igrave;/g;
    $msg =~ s//&Icirc;/g;
    $msg =~ s//&Iuml;/g;
    $msg =~ s//&Oacute;/g;
    $msg =~ s//&Ograve;/g;
    $msg =~ s//&Ocirc;/g;
    $msg =~ s//&Ouml;/g;
    $msg =~ s//&Uacute;/g;
    $msg =~ s//&Ugrave;/g;
    $msg =~ s//&Ucirc;/g;
    $msg =~ s//&Uuml;/g;
    $msg =~ s//&Ccedil;/g;
    $msg =~ s//&laquo;/g;
    $msg =~ s//&raquo;/g;
    $msg =~ s//&copy;/g;
    $msg =~ s//&reg;/g;
    $msg =~ s/\x{2013}/-/g;
    $msg =~ s/\x{2026}/&hellip;/g;
    $msg =~ s/(.)/$1/g;
    $msg =~ s/(.)/code64($1)/ge;

    return $msg; 
}

sub error # display error message and exit
{
    my $self = shift;
    my $msg = shift;

    if (!$header)
    {
	print $self->headerCode();
    }

    print "<div class=\"error\">\n";
    printf(__($msg), @_);
    print " \n\n";

    print "<form>\n";
    print '<input type="submit" value="'.__("Back").'" onclick="history.back();return false;"">'."\n";
    print "</form>\n";

    print " </div>\n";

    print $self->footerCode();
    exit;
}

sub setAlert # display alert message on page loading
{
    my $self = shift;
    my $msg = shift;

    $msg = sprintf(__($msg), @_);
    $msg =~ s/\"/\\\"/g;
    $msg =~ s/\'/&\#39;/g;

    $self->{onload} = 'alert("'.$msg.'");';
}

sub exportCode # return HTML header for a text export
{
    my $self = shift;
    my $filename = shift; # name of the file to export

    my $code = 'Content-type: application/binary; name="'.$filename.'"'."\n";
    $code .= 'Content-Disposition: inline; filename="'.$filename.'"'."\n\n";
    return $code;
}

sub writeFileUpload # write an uploaded file in a local file
{
    my $class_name = shift;
    my $field = shift; # field corresponding to uploaded file
    my $file = shift; # local file to use

    open(FILE, ">$file");
    my $size;
    my $buffer;
    my $query = new CGI;
    my $filename = $query->param($field);
    while ($size = read($filename,$buffer,1024))
    {
        print FILE $buffer;
    }
    close(FILE);
}

sub confirmCode # javascript code to display a confirm popup for a button
{
    my $self = shift;
    my $msg = shift; # i18n message to display in the confirm popup

    my $code = 'onclick=\'return confirm("';
    $msg = sprintf(__($msg), @_);
    $msg =~ s/\"/\\\"/g;
    $code .= $self->unquote($msg);
    $code .= '")\'';
    return $code;
}

sub getPage # get the current page
{
    my $self = shift;
    my $reset = shift; # to reset page if no nav cmd

    my $page = $self->{page} + 0;

    if ($self->{more})
    {
	$page = 2;
    }
    elsif ($self->{next})
    {
	$page++;
    }
    elsif ($self->{prev})
    {
	$page--;
    }
    elsif ($self->{goto})
    {
	my @gotos = split(/\0/, $self->{goto});
	my $goto;
	foreach $goto (@gotos)
	{
	    if ($goto != $page)
	    {
		return $goto;
	    }
	}   
	if ($reset)
	{
	    $page = $reset;
	}
    }
    elsif ($reset)
    {
	$page = $reset;
    }

    return $page;
}

sub pagebarCode # HTML code to display pagebar
{
    my $self = shift;
    my $page = shift; # page to display;
    my $lastpage = shift; # 1 if last page;

    if ($page > 0)
    {
	my $code = "<table class=\"pagebar\" border=0 cellspacing=0 cellpadding=0 width=\"100%\">\n";
	$code .= "<tr class=\"pagebar\">\n";
	if ($page > 1)
	{
	    $code .= '<td class="prevbutton"><input type="submit" class="prevbutton" name="prev" value="'.__("Previous page").'">'."</a></td>\n";
	}
	$code .= '<td width="100%" class="pagenumber">';
	$code .= '<div class="page">';
	$code .= '<div class="pagelabel">'.__("Page")."</div>\n";
	$code .= '<select class="pageselect" name="goto" onchange="form.submit();">'."\n";
	my $i;
	for ($i = 1; $i < $page; $i++)
	{
	    $code .= '<option value="'.$i.'">'.$i."\n";
	}
	$code .= '<option value="'.$page.'" selected>'.$page."\n";
	$code .= "</select>\n";
	$code .= "</td>\n";
	if ($lastpage)
	{
	    if ($page == 1) # only one page
	    {
		return ""; 
	    }
	}
	else
	{
	    $code .= '<td class="nextbutton"><input type="submit" class="nextbutton" name="next" value="'.__("Next page").'">'."</a></td>\n";
	}
	$code .= "</tr>\n";
	$code .= "<table>\n";
	return $code;
    }
    elsif (!$lastpage)
    {
	return '<input type="submit" class="morebutton" name="more" value="'.__("More news...").'">'."</a>\n";
    }
    else
    {
	return "";
    }
}

sub getKeywords # return keywords table from keywords string
{
    my $self = shift;
    my $backend = shift; # selected backend
    my $keystr = shift; # keywords string

    my @keywords = ();

    my @keys = $keystr =~ /\"([^\"]+)\"/g;
    $keystr =~ s/\"[^\"]+\"//g;
    my $key;
    my $mne = MN::Engine->getCurrent(); # MyNews Engine

    foreach $key (@keys)
    {
	$key = $mne->{backends}->{$backend}->{dbh}->quote($key);
	$key =~ s/^\'//;
	$key =~ s/\'$//;
	push @keywords, lc($key);
    }
    foreach $key (split(/\s+/, $keystr))
    {
	if ($key)
	{
	    $key = $mne->{backends}->{$backend}->{dbh}->quote($key);
	    $key =~ s/^\'//;
	    $key =~ s/\'$//;
	    push @keywords, lc($key);
	}
    }
    return @keywords;
}

sub _c_ # write strings in CGI console
{
    my $msg = shift;
    
    $console .= $msg;
}

sub insertCode #insert HTML code from a script or a file
{
    my $self = shift;
    my $url = shift; # url of the script/file to insert

    my $code = "";
    my $http = new HTTP::Lite();
    $http->add_req_header("Accept-Language", $currentlang);
    $http->add_req_header("User-Agent", "MyNews $MN::VERSION");
    $url .= '?__script='.$ENV{SCRIPT_NAME}.'&__lang='.$currentlang;
    if ($ENV{REMOTE_USER})
    {
	$url .= '&__user='.$ENV{REMOTE_USER}
    }
    if ($ENV{QUERY_STRING})
    {
	$url .= '&'.$ENV{QUERY_STRING}
    }
    my $req = $http->request($url);
    if ($req)
    {
        my @lines = split(/[\n\r]+/, $http->body());
	my $add = 0;
	my $line;
	foreach $line (@lines)
	{
	    if ($line =~ /<body[^>]*>/)
	    {
		$add = 1;
	    }
	    elsif ($line =~ /<\/body>/)
	    {
		$add = 0;
	    }
	    elsif ($add)
	    {
		$code .= "$line\n";
	    }
	}
    }
    return $code;
}

1;

__END__

=pod

=head1 NAME

MN::CGI - MyNews Common Gateway Interface.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 ...
 my $paramval = $cgi->{'param'};

 print $cgi->headerCode();
 print $cgi->menuCode();
 print $cgi->footerCode();

=head1 DESCRIPTION

MyNews CGI is a class that encapsulate the standard CGI module to offer special methods for MyNews web based interface.

The MN::CGI object contains a hash table with the values of the parameters of the cgi script. The current CGI object can be accessed at any time by using the B<getCurrent> class method.

=over 4

=head2 User

=item getUser ()

Get the MN::User object corresponding to the current user connected to the MyNews web based interface.

=head2 I18n

=item __ ($msg)

i18n function to get the internationalized version of a message. For example, if the current language is B<fr>, __("Search") will return "Rechercher".

=item setLang ($lang)

Set the current language to $lang. Load the corresponding messages file to internationalize interface for this language.

=item prefLang ()

Get the user preferred language from the client browser configuration.

=item getLangName ($lang)

Return the full name of the $lang language. For example, $cgi->getLangName("fr") will return "Franais".

=item langSelectorCode ([$selected], [$submit], [$css], [$name])

Return HTML code to display a language selector. $selected is the default selected language. If $submit = 1, the current form that contains the language selector is submited when the selected language is changed. $css is the suffix added to the CSS class for the selector. $name is the alternate parameter name of the selector ("lang" by default).

=head2 HTML page

=item headerCode ([$item1], [$url1], [$item2], [$url2]...)

Return HTML code for the page header. Optional parameters (couples of name/url) are used to build the page title and the navigation bar. For example, $cgi->headerCode(__("Preferences"), "/prefs.cgi") will produce a page title : "MyNews > Preferences".

=item menuCode ()

Return HTML code to display the navigation bar and the menu options.

=item adminMenuCode ()

Return HTML code for the admin menu. Called by B<menuCode> when current user has admin privilege.

=item footerCode ()

Return HTML code to end the HTML page. Displays the console if current user has admin privilege. If footerurl is defined, footerCode adds the result of this url to the returned HTML code.

=item error ($msg, [params])

Stop the current script execution and display an error message. $msg is internationalized and can have parameters. Example: $cgi->error("Template %s doesn't exist", $cgi->{id})

=item exportCode ($filename)

Return HTTP header to generate a binary file download (for example to export data). $filename is the name of the binary file that will be downloaded.

=head2 Javascript

=item setAlert ($msg, [params])

Set a message to be displayed in aa alert popup window when displaying the HTML page. Must be called before B<headerCode>. $msg is internationalized and can have parameters. Example: $cgi->setAlert("Template %s doesn't exist", $cgi->{id})

=item confirmCode ($msg, [params])

Return the code to display a confirm popup window with a message when clicking on some elements (buttons, links...). $msg is internationalized and can have parameters. Example: $cgi->confirmCode("Do you really want to delete template %s?", $cgi->{id})

=head2 Pagebar

=item getPage ([$reset])

Return the current page calculated from navigation commands (next, prev...). If $reset is set, the current page is forced to the value of $reset if no navigation command is set.

=item pagebarCode ($page, [$lastpage])

Return the HTML code of the pagebar for the page $page. If $lastpage is set to 1, don't display the "next" button.

=head2 Search

=item getKeywords ($keystr)

Return a table of keywords from the keyword string $keystr. A keyword can contains space characters and must be surrounded by "". Example : $cgi->getKeywords('key1 "key2 with space" key3') will return ("key1", "key2 with space", "key3).

=head2 Misc

=item unquote ($msg)

Return a $msg where some characters (',",<,>,&) has been unquoted to avoid HTML problems.

=item unutf ($msg)

Return a $msg where some UTF characters has been replaced by ISO-8859-1 equivalent.

=item writeFileUpload ($field, $file)

Write the uploaded file contained in the param $field in the local file called $file.

=item _c_ ($msg)

Write the message $msg in the console. For example _c_("Debug message...")

=item insertCode ($url)

Get the result code of the URL $url (only the body, no head). Current QUERY_STRING is added to the URL, current script name is passed as __script parameter and current user (if set) is passed ans __user parameter.

=head1 SEE ALSO

L<CGI>, L<MN::User>, L<MN::Theme>
