package MN::Template; # class to manage MyNews templates

use strict;

@MN::Template::ISA = qw(MN::Object);

use MN::Object;
use MN::News;
use MN::Topic;
use HTTP::Lite;
use XML::RSS::Parser;

my %current = ();
my %updates = ();

sub getCurrent # get current template hash
{
    return \%current;
}

sub setCurrent # put the template in the current hash table
{
    my $self = shift;
    $current{$self->{id}} = $self;
}

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new Template object
}
sub getClass # get database table for the class 
{
    return "mntemplate"; # "mntemplate"
}

sub getKey # get database key for the class
{
    return "id"; # "id"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"id" => "string",
	"name" => "string",
	"url" => "string",
	"comment" => "string",
	"lang" => "string",
	"active" => "number",
	"rx_news_id" => "string",
	"rx_news_name" => "string",
	"rx_news_start" => "string",
	"rx_news_end" => "string",
	"rx_news_date" => "string",
	"rx_news_first" => "string",
	"rx_news_last" => "string",
	"rx_topic_id" => "string",
	"rx_topic_name" => "string",
	"rx_topic_icon" => "string",
	"link_news" => "string",
	"link_icon" => "string",
	"rx_search_strings" => "string",
	"rx_replace_strings" => "string",
	"icon_url" => "string",
	"mnid" => "string",
	"mntm" => "number",
    };
}

sub getHTMLPage # get the HTML page from news website
{
    my $self = shift;
    my $test = shift; # test mode
    my $cache = shift; # cache mode

    if ($cache && -e "/tmp/$self->{id}.html")
    {
	open(FILE, "/tmp/$self->{id}.html");
	my @lines = <FILE>;
	my $lines = join("", @lines);
	if ($lines =~ /<\?xml/)
	{
	    $self->{http_type} = "text/xml";
	}
	return $lines;
    }
    else
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $self->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request($self->{url});
	if ($req)
	{
	    if ($test)
	    {
		open(FILE, ">/tmp/$self->{id}.html");
		print FILE $http->body();
		close(FILE);
	    }
	    $self->{http_code} = $req;
	    my $lines = $http->body();
	    if ($lines =~ /<\?xml/)
	    {
		$self->{http_type} = "text/xml";
	    }
	    else
	    {
		$self->{http_type} = $http->{'resp-headers'}->{'Content-Type'}[0];
	    }
	    return $lines;
	}
	else
	{
	    return undef;
	}
    }
}

sub getNews # extract news from html page
{
    my $self = shift;
    my $idonly = shift; # 1 to only extract news id
    my $test = shift; # 1 if we are in test mode
    my $cache = shift; # 1 if we are in cache mode

    $self->{src} = $self->getHTMLPage($test, $cache);

    my @news = ();
    my $news;
    my $value;

    if ($self->{http_type} =~ /xml/) # RSS file
    {
	my $p = new XML::RSS::Parser();
	my $feed = $p->parse($self->{src});
	my $i;
	my $img;
	foreach $i ($feed->items()) 
	{
	    $news = new MN::News();
	    $news->{item} = $i;
	    $news->{template} = $self->{id};
	    $news->extract($idonly);
	    push @news, $news;
	}
	if ($test && $news)
	{
	    my $code = "";
	    if (!$self->{link_news} && $news->{link})
	    {
		if ($news->{guid})
		{
		    $self->{link_news} = $news->{link};
		    $self->{link_news} =~ s/$news->{guid}/\%s/;
		}
		else
		{
		    $self->{link_news} = '%s';
		}
		$value = $self->{link_news};
		$value =~ s/\"/\\\"/g;
		$value =~ s/\'/\\\'/g;
		$code .= "window.document.forms[0].link_news.value = '$value'\n";
	    }
	    if (!$self->{name} || !$self->{comment})
	    {
		my $channel = $feed->channel();
		my $field;
		foreach $field ($channel->children())
		{
		    if ($field->{name} eq 'title' && !$self->{name})
		    {
			$self->{name} = $field->{value};
			$value = $field->{value};
			$value =~ s/\"/\\\"/g;
			$value =~ s/\'/\\\'/g;
			$code .=  "window.document.forms[0].name.value = '$value'\n";
		    }
		    elsif ($field->{name} eq 'description' && !$self->{comment})
		    {
			$self->{comment} = $field->{value};
			$value = $field->{value};
			$value =~ s/\"/\\\"/g;
			$value =~ s/\'/\\\'/g;
			$code .=  "window.document.forms[0].comment.value = '$value'\n";
		    }
		}
	    }
	    if (!$self->{icon_url} && ($img = $feed->image()))
	    {
		my $field;
		foreach $field ($img->children())
		{
		    if ($field->{name} eq 'url')
		    {
			$self->{icon_url} = $field->{value};
			$value = $field->{value};
			$value =~ s/\"/\\\"/g;
			$value =~ s/\'/\\\'/g;
			$code .=  "window.document.forms[0].icon_url.value = '$value'\n";
		    }
		}
	    }
	    if ($code && $ENV{GATEWAY_INTERFACE})
	    {
		print "<script language=javascript>\n";
		print "<!--\n";
		print $code;
		print "//-->\n";
		print "</script>\n";
	    }
	}
    }
    else
    {
	my $line;
	my $line2;
	my $nlines = [];
	my $active;
	if (!$self->{rx_news_start})
	{
	    die "Template $self->{id} : empty rx_news_start field";
	}
	if (!$self->{rx_news_end})
	{
	    die "Template $self->{id} : empty rx_news_start field";
	}
	my @lines = split(/[\n\r]+/, $self->{src});
	$line = shift(@lines);
	$line2 = "";
	while (defined $line)
	{
	    if (!$active && $line =~ /$self->{rx_news_start}/)
	    {
		$active = 1;
	    }
	    if ($active && $line =~ /$self->{rx_news_end}/)
	    {
		if ($1)
		{
		    push @{$nlines}, $1;
		    $line2 = $line;
		    $line2 =~ s/$self->{rx_news_end}//;
		}
		else
		{
		    push @{$nlines}, $line;
		}
		$active = 0;
		$news = new MN::News();
		$news->{lines} = $nlines;
		$news->{template} = $self->{id};
		$news->extract($idonly);
		push @news, $news;
		$nlines = [];
	    }
	    if ($active)
	    {
		push @{$nlines}, $line;
	    }
	    if ($line2)
	    {
		$line = $line2;
		$line2 = "";
	    }
	    else
	    {
		$line = shift(@lines);
	    }
	}
    }
    if (!scalar(@news) && $self->{active})
    {
	return undef;
    }
    @news = reverse(@news);
    return \@news; # table of news objects
}

sub getTemplate # get news template by its name
{
    my $self = shift;
    my $name = shift; # name of the template

    if (!$name)
    {
	return undef;
    }

    if (!$current{$name})
    {
	$current{$name} = MN::Template->getObject($name);
    }
    return $current{$name};
}

sub getTopics # get all topics of a template
{
    my $self = shift;

    my @topics =  MN::Topic->getObjects("id like '$self->{id}%'", "id");
    my $topic;
    foreach $topic (@topics)
    {
	MN::Topic::getCurrent()->{$topic->{id}} = $topic;
    }
    return @topics;
}

sub getSearchStrings # get the search strings to be replaced for the template
{
    my $self = shift;

    if (!$self->{search})
    {
	my @ss = split(/[\r\n]+/, $self->{rx_search_strings});
	my @rs = split(/[\r\n]+/, $self->{rx_replace_strings});
	my %replace;
	for (my $i = 0; $i < scalar(@ss); $i++)
	{
	    $replace{$ss[$i]} = $rs[$i];
	}
	$self->{search} = \@ss;
	$self->{replace} = \%replace;
    }
    return $self->{search}; # table of search strings
}

sub getCode # get HTML code for the template
{
    my $self = shift;
    my $css = shift;
    my $url = shift;
    my $test = shift; # 1 to display template in test mode

    my $code = "<div class=\"newstemplate$css\" title=\"$self->{name} : $self->{comment}\">\n";
    my $imgdir;
    my $imgurl;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir})
    {
	$imgdir = "$mne->{conf}->{webdir}/img/templates";
    }
    if ($url || $mne->{conf}->{weburl})
    {
	$imgurl = "$mne->{conf}->{weburl}/img/templates";
	if (!$url)
	{
	    $url = $mne->{conf}->{weburl}.'/template.cgi?id='.$self->{id};
	}
	$code .= '<a href="'.$url.'" class="newstemplate'.$css.'">'."\n";
    }
    if ($test && $self->{icon_url})
    {
	$code .= '<img src="'.$self->{icon_url}.'" border=0 alt="'.$self->{name}.' : '.$self->{comment}.'" title="'.$self->{name}.' : '.$self->{comment}.'">'."\n";
    }
    elsif ($imgurl && $imgdir && -e "$imgdir/$self->{id}.png")
    {
	$code .= '<img src="'.$imgurl.'/'.$self->{id}.'.png" border=0 alt="'.$self->{name}.' : '.$self->{comment}.'" title="'.$self->{name}.' : '.$self->{comment}.'">'."\n";
    }
    else
    {
	$code .= $self->{name}."\n";
    }
    if ($url)
    {
	$code .= "</a>\n";
    }
    $code .= "</div>\n";

    return $code; # template HTML code
}

sub getTemplateIcon # extract template icon from template website
{
    my $self = shift;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $imgdir;
    if ($mne->{conf}->{webdir})
    {
	$imgdir = "$mne->{conf}->{webdir}/img/templates";
    }
    if (!$imgdir)
    {
	return 0;
    }
    if ($self->{icon_url})
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $self->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request($self->{icon_url});
	if ($req && $req == 200)
	{
	    open(IMG, ">$imgdir/$self->{id}.png");
	    print IMG $http->body();
	    close(IMG);
	    chmod(0664, "$imgdir/$self->{id}.png");
	    return 0;
	}
	else
	{
	    my $msg = $http->status_message();
	    $msg =~ s/[\r\n]+//g;
	    return $msg;
	}
    }
    else
    {
	return 0;
    }
}

sub getLastUpdate # get the date of the last template update from MyNews website
{
    my $self = shift;

    my $date = "";

    if ($self->{mntm})
    {
	my @tm = localtime($self->{mntm});
	if ($::MN::CGI::user && $::MN::CGI::user->{lang} eq "fr")
	{
	    $date = sprintf("%02d/%02d/%04d %02d:%02d:%02d", $tm[3], $tm[4]+1, $tm[5]+1900, $tm[2], $tm[1], $tm[0]);
	}
	else
	{
	    $date = sprintf("%02d/%02d/%04d %02d:%02d:%02d", $tm[4]+1, $tm[3], $tm[5]+1900, $tm[2], $tm[1], $tm[0]);
	}
	
    }
    return $date;
}

sub getStatusCode # get HTML code for the template status
{
    my $self = shift;
    my $css = shift;

    my $val = $self->{active} ? "on" : "off";
    my $msg = $self->{active} ? MN::__("Disable") : MN::__("Enable");

    my $mne = MN::Engine::getCurrent(); # MyNews Engine

    my $code = "<div class=\"status$val$css\" title=\"$msg\">\n";
    $code .= "<img class=\"imgstatus$val$css\" src=\"$mne->{conf}->{weburl}/img/$val.png\" alt=\"$msg\" title=\"$msg\" border=0>\n";
    $code .= "</div>\n";

    return $code;
}

sub getExportCode # get text code to export a template
{
    my $self = shift;
    
    my $code = "";
    my $fields = $self->getFields();
    my $field;
    my $txt;
    foreach $field (sort(keys(%{$fields})))
    {
	if ($field =~ /strings$/)
	{
	    $code .= "start\t\t$field\n";
	    $txt = $self->{$field};
	    $txt =~ s/\r//mg;
	    if (length($txt))
	    {
		$code .= "$txt\n";
	    }
	    $code .= "end\t\t$field\n";
	}
	else
	{
	    $code .= "$field\t\t$self->{$field}\n";
	}
    }
    return $code;
}

sub getMyNewsUpdates # get tmpls update time from MyNews website
{
    my $self = shift;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{mnurl})
    {
	my $http = new HTTP::Lite();
	if ($ENV{REMOTE_USER})
	{
	    $http->add_req_header("Accept-Language", MN::CGI->getUser()->{lang});
	}
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request("$mne->{conf}->{mnurl}/templates/update.php");
	if ($req)
	{
	    my $line;
	    foreach $line (split(/\n/,$http->body()))
	    {
		if ($line =~ /^(\S+)\s*(\d+)$/)
		{
		    $updates{$1} = $2;
		}
	    }
	}
    }
}

sub getUpdateCode # get HTML code for the template status
{
    my $self = shift;
    my $css = shift;

    my $val;
    my $code = "";
    my $msg;

    if ($self->{mnid} && $updates{$self->{mnid}})
    {
	my $mne = MN::Engine::getCurrent(); # MyNews Engine

	if ($updates{$self->{mnid}} > $self->{mntm})
	{
	    $msg = MN::__("Needs update");
	    $code .= '<a href="templates.cgi?update='.$self->{mnid}.'"><div class="statusoff'.$css.'" title="'.$msg.'">'."\n";
	    $code .= "<img class=\"imgstatusoff$css\" src=\"$mne->{conf}->{weburl}/img/off.png\" alt=\"$msg\" title=\"$msg\" border=0>\n";
	    $code .= "</div></a>\n";
	}
	else
	{
	    $msg = MN::__("Up-to-date");
	    $code .= '<div class="statuson'.$css.'" title="'.$msg.'">'."\n";
	    $code .= "<img class=\"imgstatuson$css\" src=\"$mne->{conf}->{weburl}/img/on.png\" alt=\"$msg\" title=\"$msg\" border=0>\n";
	    $code .= "</div>\n";
	}
    }

    return $code;
}

sub updateFromMyNews # update template from MyNews website
{
    my $self = shift;
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{mnurl})
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $self->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request("$mne->{conf}->{mnurl}/templates/update.php?id=$self->{mnid}");
	if ($req)
	{
	    my $mntfile = "/tmp/$ENV{REMOTE_USER}.mnt";
	    open(MNT, ">$mntfile");
	    print MNT $http->body();
	    close(MNT);
	    $self->updateFromFile($mntfile);
	    unlink($mntfile);
	    $self->{mntm} = time();
	}
    }
}

sub updateFromFile # update template from .mnt file
{
    my $self = shift;
    my $mntfile = shift; # .mnt file

    my $tid = $self->{id};
    my $line;
    my $key;
    my $val;
    my $mlf = 0;
    my $str = "";
    my $sep = "";
    open(MNT, $mntfile);
    while ($line = <MNT>)
    {
	chomp($line);
	($key,$val) = split(/\t+/, $line);
	if ($mlf)
	{
	    if ($key eq 'end')
	    {
		$self->{$mlf} = $str;
		$str = "";
		$sep = "";
		$mlf = 0;
	    }
	    else
	    {
		$str .= $sep.$line;
		$sep = "\n";
	    }
	}
	elsif ($key eq 'start')
	{
	    $mlf = $val;
	    $str = "";
	    $sep = "";
	}
	else
	{
	    $self->{$key} = $val;
	}
    }
    close(MNT);
    if ($tid)
    {
	$self->{id} = $tid;
    }
}

sub delete # database destructor
{
    my $self = shift;

    # delete topics
    my @topics = $self->getTopics();
    my $topic;
    foreach $topic (@topics)
    {
	$topic->delete();
    }

    # delete icon
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir})
    {
	my $iconfile = "$mne->{conf}->{webdir}/img/templates/$self->{id}.png";
	if (-e $iconfile)
	{
	    unlink($iconfile);
	}
    }

    # delete news
    $self->execSQL("delete from mnnews where template='$self->{id}'");

    $self->SUPER::delete();
}

sub getNewTemplates # get list of new templates on MyNews website
{
    my $self = shift;

    my @tmpls = ();
    my $tmpl;
    foreach $tmpl (keys(%updates))
    {
	if (!$current{$tmpl})
	{
	    push @tmpls, $tmpl;
	}
    }
    return sort @tmpls;
}

sub getFlag # get HTML to display flag of the template
{
    my $self = shift;
    my $css = shift;

    if (!$self->{flag})
    {
	my $mne = MN::Engine->getCurrent(); # MyNews Engine
	open(FILE, "$mne->{conf}->{webdir}/lang/$self->{lang}/name");
	my $name = <FILE>;
	chomp($name);
	close(FILE);
	$self->{flag} = "<img src=\"$mne->{conf}->{weburl}/img/flags/$self->{lang}.png\" class=\"flag$css\" alt=\"$name\" title=\"$name\">";
    }
    return $self->{flag};
}

sub getCurrentCondition # get SQL condition to select news from current templates
{
    my $self = shift;

    return "template in ('".join("','", keys(%current))."')";
}

1;

__END__

=pod

=head1 NAME

MN::Template - MyNews class to manage templates.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $template = MN::Template->getObject("slashdot");
 ...

=head1 DESCRIPTION

MyNews Template objects store informations about website templates.

MN::Template inherits from MN::Object, that means that template objects are stored in database.

=over 4

=head2 Fields

=item id : string

Template id (primary key). For example: "slashdot".

=item name : string

Template name (displayed as alternate text to template icon). For example: "Slashdot".

=item url : string

URL of the website to scan. For example: "http://slashdot.org/".

=item icon_url : string

URL of the template icon in the website. For example: "http://images.slashdot.org/topics/topicslashdot.gif".

=item comment : string

Free text to describe the website. For example: "News for nerds, stuff that matters".

=item active : number

Status of the template (1: active, 0: inactive).

=item rx_news_id : string

Regular expression to extract news id.

=item rx_news_name : string

Regular expression to extract news name.

=item rx_news_date : string

Regular expression to extract news date.

=item rx_news_start : string

Regular expression to detect the beginning of the news code.

=item rx_news_end : string

Regular expression to detect the end of the news code.

=item rx_news_first : string

Regular expression to detect the first line of the news body.

=item rx_news_last : string

Regular expression to detect the last line of the news body.

=item rx_topic_id : string

Regular expression to extract topic id.

=item rx_topic_name : string

Regular expression to extract topic name.

=item rx_topic_icon : string

Regular expression to extract topic icon file.

=item rx_search_strings : string

Regular expression to define strings to search in the news body.

=item rx_replace_strings : string

Regular expression to define strings to replace in the news body.

=item link_news : string

URL of the news in the template website (%s will be replaced by the news id, if link_news contains 2 %s, the first one is replaced by the topic id and the second one by the news id).

=item link_icon : string

URL of the topic icon in the template website (%s will be replaced by the topic iconfile).

=item mnid : string

Id of the template in the MyNews distant website (for template update purpose).

=item mntm : string

Date of the last template update from the MyNews distant website.

=head2 Template information

=item getTemplate ($id)

Return template object by its id. Avoid database access if the template object has been already selected.

=item getTopics ()

Return all topics of the template in a table.

=item getSearchStrings ()

Return a hash table of type (search string => replace string) for the template substitutions in news body.

=item getCode ([$css], [$url], [$test])

Return the HTML code to display the template in the MyNews website.

$css is the suffix to add to the template CSS class.

$url is the alternate URL of the link used when clicking on template icon.

If $test is set to 1, display the template in test mode (use remote icon).

=item getStatusCode ([$css])

Return the HTML code to display the template status.

$css is the suffix to add to the template status CSS class.

=item getFlag ([$css])

Return the HTML code to display the template flag.

$css is the suffix to add to the template flag CSS class.

=head2 Template extraction

=item getTemplateIcon ()

Extract icon of the template.

=head2 Template import/export

=item getExportCode ()

Return the text representation of a template for exportation purpose (.mnt file).

=item updateFromFile ($mntfile)

Update a template with the text representation in the $mntfile file.

=head2 Template update from MyNews distant site

=item getMyNewsUpdates ()

Get templates update time from MyNews distant site.

=item getUpdateCode ([$css])

Get the update status of the template (Needs update or Up-to-date). The B<getMyNewsUpdates> method must have been called before calling this method.

$css is the suffix to add to the update status CSS class.

=item updateFromMyNews ()

Update a template from MyNews distant site. Change B<mntm> field value.

=item getNewTemplates ()

Get the new templates in the MyNews distant site not present in the MyNews local site. The B<getMyNewsUpdates> method must have been called before calling this method.

=head1 SEE ALSO

L<MN::Object>, L<MN::Template>, L<MN::News>
